<?php

namespace App\Http\Services\addon\committee;

use App\Models\Committee;
use App\Models\FileManager;
use App\Models\User;
use App\Traits\ResponseTrait;
use Exception;
use Illuminate\Support\Facades\DB;

class CommitteeService
{
    use ResponseTrait;

    public function list($request)
    {
        $committee = Committee::where('committees.tenant_id', getTenantId())
            ->join('committee_designations', 'committee_designations.id' , '=', 'committees.committee_designation_id')
            ->join('committee_categories', 'committee_categories.id' , '=', 'committees.committee_category_id')
            ->join('committee_elections', 'committee_elections.id' , '=', 'committees.committee_election_id')
            ->with('user')
            ->select('committee_designations.name as designation','committee_categories.name as category', 'committee_designations.order', 'committee_elections.session', 'committees.*')
            ->orderBy('committee_elections.id', 'DESC')
            ->orderBy('committee_designations.order', 'ASC');

        if($request->committee_election_id != 0){
            $committee->where('committee_elections.id', $request->committee_election_id);
        }
        if($request->committee_category_id != 0){
            $committee->where('committee_categories.id', $request->committee_category_id);
        }

        return datatables($committee)
            ->addIndexColumn()
            ->editColumn('photo', function ($data) {
                if(!is_null($data->photo)){
                    return '<img class="avatar-xs max-h-35 max-h-69 rounded" src="'.getFileUrl($data->photo) .'"/>';
                }
                return '<img class="avatar-xs max-h-35 max-h-69 rounded" src="'.getFileUrl($data->user->image) .'"/>';
            })

            ->editColumn('status', function ($data) {
                if($data->status == STATUS_DEACTIVATE) {
                    return '<span class="zBadge-free">'.__("Deactivated").'</span>';
                }

                return '<span class="zBadge-one">'.__("Active").'</span>';
            })
            ->addColumn('action', function ($data) {
                return '<ul class="d-flex align-items-center cg-5 justify-content-center">
                            <li class="d-flex gap-2">
                                <button onclick="getEditModal(\'' . route('committee.admin.committee.info', [$data->id]) . '\'' . ', \'#edit-modal\')" class="d-flex justify-content-center align-items-center w-30 h-30 rounded-circle bd-one bd-c-ededed bg-white" title="' . __('Edit') . '">
                                    <img src="' . asset('assets/images/icon/edit.svg') . '" alt="edit" />
                                </button>
                                <button onclick="deleteItem(\'' . route('committee.admin.committee.delete', [$data->id]) . '\', \'committeeDatatable\')" class="d-flex justify-content-center align-items-center w-30 h-30 rounded-circle bd-one bd-c-ededed bg-white" title="' . __('Delete') . '">
                                    <img src="' . asset('assets/images/icon/delete-1.svg') . '" alt="delete">
                                </button>
                            </li>
                        </ul>';
            })
            ->rawColumns(['action', 'photo', 'status'])
            ->make(true);
    }

    public function store($request)
    {
        DB::beginTransaction();
        try {
            $photo = NULL;

            if ($request->hasFile('photo')) {
                $new_file = new FileManager();
                $uploaded = $new_file->upload('election', $request->photo);
                $photo = $uploaded->id;
            }

            $userData = User::join('alumnus','alumnus.user_id','users.id')
                ->select(
                    'alumnus.company as alumnusCompanyName',
                    'alumnus.company_address as alumnusCompanyAddress'
                )
                ->where('users.tenant_id',getTenantId())
                ->where('users.id',$request->user_id)
                ->where('alumnus.user_id',$request->user_id)->first();

            $createData = [
                'tenant_id' => getTenantId(),
                'committee_election_id' => $request->committee_election_id,
                'committee_category_id' => $request->committee_category_id,
                'user_id' => $request->user_id,
                'committee_designation_id' => $request->designation_id,
                'photo' => $photo,
                'company' => $userData->alumnusCompanyName,
                'address' => $userData->alumnusCompanyAddress,
                'created_by' => auth()->id(),
                'status' => $request->status,
            ];

            Committee::create($createData);
            DB::commit();
            return $this->success([], getMessage(CREATED_SUCCESSFULLY));
        } catch (Exception $e) {
            DB::rollBack();
            return $this->error([], getMessage(SOMETHING_WENT_WRONG));
        }
    }

    public function update($request, $id)
    {
        DB::beginTransaction();
        try {
            $committee = Committee::where('id', $id)->with('user')->first();
            $photo = $committee->photo;

            if ($request->hasFile('photo')) {
                $new_file = new FileManager();
                $uploaded = $new_file->upload('election', $request->photo);
                $photo = $uploaded->id;
            }

            $userData = User::join('alumnus','alumnus.user_id','users.id')
                ->select(
                    'alumnus.company as alumnusCompanyName',
                    'alumnus.company_address as alumnusCompanyAddress'
                )
                ->where('users.tenant_id',getTenantId())
                ->where('users.id',$request->user_id)
                ->where('alumnus.user_id',$request->user_id)->first();

            $updateData = [
                'tenant_id' => getTenantId(),
                'committee_election_id' => $request->committee_election_id,
                'committee_category_id' => $request->committee_category_id,
                'user_id' => $request->user_id,
                'committee_designation_id' => $request->designation_id,
                'photo' => $photo,
                'company' => $userData->alumnusCompanyName,
                'address' => $userData->alumnusCompanyAddress,
                'status' => $request->status,
            ];

            $committee->update($updateData);
            DB::commit();
            return $this->success([], getMessage(UPDATED_SUCCESSFULLY));
        } catch (Exception $e) {
            DB::rollBack();
            return $this->error([], getMessage(SOMETHING_WENT_WRONG));
        }
    }

    public function getById($id)
    {
        return Committee::where('tenant_id', getTenantId())->where(['id' => $id])->first();
    }

    public function deleteById($id)
    {
        DB::beginTransaction();
        try {
            $committee = $this->getById($id);
            $committee->user->update(['status', STATUS_DEACTIVATE]);
            $committee->delete();
            DB::commit();
            $message = getMessage(DELETED_SUCCESSFULLY);
            return $this->success([], $message);
        } catch (\Exception $e) {
            DB::rollBack();
            $message = getErrorMessage($e, $e->getMessage());
            return $this->error([], $message);
        }
    }
}
